/********************************************************************************************
 *  Ebbie-MM Version 4.2
 *  Copyright (c)  2008 Nov.  H. Alexander Ebhardt, Herbert H. Tsang and Denny Chen Dai
 *  Permission is granted to copy, distribute and/or modify this document
 *  under the terms of the GNU Free Documentation License, Version 1.2
 *  or any later version published by the Free Software Foundation;
 *  with no Invariant Sections, no Front-Cover Texts, and no Back-Cover
 *  Texts.  A copy of the license is included in the section entitled "GNU
 *  Free Documentation License".
 *********************************************************************************************/

package ebbieMM;

import com.sun.corba.se.impl.orbutil.concurrent.Mutex;
import ebbieMM.fasta.Fasta;
import java.util.Vector;

/**
 *
 * @author Denny Chen Dai
 */
public class BlastAlgs {
        
    private BlastNSpawner spw = null;
    private Mutex mtx = null;
    Vector query_buffer = null;
    Vector result_buffer = null;
     
    public BlastAlgs(String dbF, boolean setPenalty, float penalty){
        spw = new BlastNSpawner(dbF);
        mtx = new Mutex();
        query_buffer = new Vector();
        result_buffer = new Vector();    
        /*initialize the blast spawner*/
        spw.spawnBlastN(query_buffer, result_buffer, mtx, setPenalty, penalty);
    }
    
    /**
     * Terminate the Blast spawner process
     */
    public void terminate(){
        try{
            mtx.acquire();
            query_buffer.clear();
            result_buffer.clear();
            query_buffer.add("@NN"); //termination symbol
            mtx.release();
        }catch(InterruptedException e){;}
   
                
        /*wait for eater exist*/
/*        while(true){
            if(spw.eater_flag.size() > 0){
                try{         
                    Thread.currentThread().sleep(1000);}
                catch(InterruptedException e){;}       
            }
            else
                break;
        }
*/      
        /*     
        if(p!=null){
            p.destroy();
            System.out.println("Terminating BlastN...exit value: " + p.exitValue());
        }
         */
    }

    /**
     * run blast against given query sequence
     * and return the blast output in string
     * 
     * @param The query seqence in fasta format 
     * @return the blast output stream
     */
    public String blast_query(Fasta query){
        
        /*clear buffer*/
        try{
            mtx.acquire();        
            query_buffer.clear();
            result_buffer.clear();
            mtx.release();
        }
        catch(InterruptedException e){;}
        
        /*pump single sequece out for blasting*/
        String seq = query.sequence;
        try{
            mtx.acquire();
            query_buffer.add(seq);
            mtx.release();
        }catch(InterruptedException e)
        {
            System.err.println("Mutex error in blast_query");
        }

        /*wait for blast result*/
        while(true){
            if(result_buffer.size() < 1){
                try{         
                    Thread.sleep(100);}
                catch(InterruptedException e){;}       
            }
            else
                break;
        }
        
        /*take the blast result and return*/
        String result = "";
        try{
            mtx.acquire();
            result = (String)result_buffer.remove(0); 
            mtx.release();
        }catch(InterruptedException e){;}
        
        //System.out.println("Receiving Blast output");
        return result;
    }

}








